@@ -24,24 +24,38 @@ GRANT_TYPE_MAPPING = {
 
 class OAuthValidator(RequestValidator):
 
-    def authenticate_client(self, request, *args, **kwargs):
+    def _authenticate_client_basic(self, request):
         """
-        Try to authenticate the client.
+        Try authenticating the client using HTTP Basic Authentication method
         """
         auth = request.headers.get('HTTP_AUTHORIZATION', None)
-        client_id = request.body.get('client_id', None)
-        client_secret = request.body.get('client_secret', None)
 
-        if auth and not client_id and not client_secret:
-            auth_type, auth_string = auth.split(' ')
-            encoding = request.encoding or 'utf-8'
+        if not auth:
+            return False
 
-            try:
-                auth_string_decoded = base64.b64decode(auth_string).decode(encoding)
-                client_id, client_secret = auth_string_decoded.split(':', 1)
-            except (TypeError, UnicodeDecodeError):
-                return False
-        elif not client_id and not client_secret:
+        auth_type, auth_string = auth.split(' ')
+        if not auth_type == 'Basic':
+            return False
+
+        encoding = request.encoding or 'utf-8'
+
+        auth_string_decoded = base64.b64decode(auth_string).decode(encoding)
+        client_id, client_secret = auth_string_decoded.split(':', 1)
+
+        try:
+            request.client = Application.objects.get(client_id=client_id, client_secret=client_secret)
+            return True
+        except Application.DoesNotExist:
+            return False
+
+    def _authenticate_client_body(self, request):
+        """
+        Try authenticating the client using values from request body
+        """
+        client_id = request.client_id
+        client_secret = request.client_secret
+
+        if not client_id:
             return False
 
         try:
@@ -50,6 +64,17 @@ class OAuthValidator(RequestValidator):
         except Application.DoesNotExist:
             return False
 
+    def authenticate_client(self, request, *args, **kwargs):
+        """
+        Try to authenticate the client.
+        """
+        authenticated = self._authenticate_client_basic(request)
+
+        if not authenticated:
+            authenticated = self._authenticate_client_body(request)
+
+        return authenticated
+
     def authenticate_client_id(self, client_id, request, *args, **kwargs):
         """
         Ensure client_id belong to a non-confidential client.
@@ -86,8 +111,7 @@ class OAuthValidator(RequestValidator):
         """
         Get the list of scopes associated with the refresh token.
         """
-        rt = RefreshToken.objects.get(token=refresh_token)
-        return rt.access_token.scope
+        return request.refresh_token_object.access_token.scope
 
     def invalidate_authorization_code(self, client_id, code, request, *args, **kwargs):
         """
@@ -111,6 +135,14 @@ class OAuthValidator(RequestValidator):
         """
         Persist the Bearer token.
         """
+        if request.refresh_token_object:
+            # Remove used refresh token
+            try:
+                RefreshToken.objects.get(token=request.refresh_token).delete()
+            except RefreshToken.DoesNotExist:
+                # Already deleted?
+                assert()
+
         expires = timezone.now() + timedelta(seconds=oauth_api_settings.ACCESS_TOKEN_EXPIRATION)
         if request.grant_type == 'client_credentials':
             request.user = request.client.user
@@ -124,9 +156,6 @@ class OAuthValidator(RequestValidator):
         access_token.save()
 
         if 'refresh_token' in token:
-            # Discard old refresh token
-            RefreshToken.objects.filter(user=request.user, application=request.client).delete()
-
             refresh_token = RefreshToken(
                 user=request.user,
                 token=token['refresh_token'],
@@ -198,6 +227,7 @@ class OAuthValidator(RequestValidator):
         try:
             rt = RefreshToken.objects.select_related('user').get(token=refresh_token)
             request.user = rt.user
+            request.refresh_token_object = rt
             return rt.application == client
         except RefreshToken.DoesNotExist:
             return False
